from influxdb_client import InfluxDBClient
import matplotlib.pyplot as plt
import pandas as pd
import math
import json

# Load configuration from config.json
config_path = r"C:\Allianz\4\0211\1201\config.json"
with open(config_path, 'r') as config_file:
    config = json.load(config_file)

url = config["url"]
token = config["token"]
org = config["org"]
bucket = config["bucket"]

try:

    # Create an InfluxDB client
    client = InfluxDBClient(url=url, token=token, org=org)

    # Query data from InfluxDB
    query = f'from(bucket:"{bucket}") |> range(start: 2023-08-01T00:00:00Z, stop: 2023-08-02T00:00:00Z)'
    tables = client.query_api().query(query, org=org)

    # Extract points from the result
    extracted_influx_objects = {}
    for table in tables:
        for record in table.records:
            url_hostname = record.values.get("url_hostname")
            if url_hostname:
                extracted_influx_objects[url_hostname] = extracted_influx_objects.get(url_hostname, 0) + 1

    # Convert the data to a pandas DataFrame
    df = pd.DataFrame(list(extracted_influx_objects.items()), columns=["URL Hostname", "Request Count"])

    # Sort by request count and get the top 20 URLs
    df_top_20 = df.sort_values(by="Request Count", ascending=False).head(20)

    # Create aliases for the URLs
    df_top_20["URL Alias"] = [f"URL {i+1}" for i in range(len(df_top_20))]

    # Determine the maximum request count for scaling
    max_count = df_top_20["Request Count"].max()
    max_log_scale = math.ceil(math.log10(max_count))  # Find the next power of 10
    y_ticks = [10**i for i in range(1, max_log_scale + 1)]  # Adjust ticks dynamically

    # Create a figure with two subplots (Logarithmic and Linear Scale)
    fig, (ax1, ax2) = plt.subplots(1, 2, figsize=(16, 7))

    # Logarithmic scale plot
    ax1.bar(df_top_20["URL Alias"], df_top_20["Request Count"], color="skyblue")
    ax1.set_yscale("log")
    ax1.set_yticks(y_ticks)
    ax1.set_yticklabels([f"$10^{i}$" for i in range(1, max_log_scale + 1)])  # Update y-tick labels
    ax1.set_ylabel("Request Counts (Log Scale)", fontsize=12)
    ax1.set_title("Top 20 URLs by Request Count (Log Scale)", fontsize=14)
    ax1.tick_params(axis="x", which="both", length=0)  # Remove x-axis ticks and labels

    # Linear scale plot
    ax2.bar(df_top_20["URL Alias"], df_top_20["Request Count"], color="skyblue")
    ax2.set_ylabel("Request Counts (Linear Scale)", fontsize=12)
    ax2.set_title("Top 20 URLs by Request Count (Linear Scale)", fontsize=14)
    ax2.tick_params(axis="x", which="both", length=0)  # Remove x-axis ticks and labels

    # Layout adjustments and show plot
    plt.tight_layout()
    plt.show()

except Exception as e:
    print(f"An error occurred: {e}")
